"""Helpers library for zet"""

import os
import sys
import shutil
import tempfile
import contextlib
from functools import cache


def die(msg=None, status=1):
    if msg:
        eprint(msg)
    sys.exit(status)


def eprint(*a, **kw):
    kw["file"] = sys.stderr
    print(*a, **kw)


def getenv(varname):
    """Get the value of environment variable"""
    return os.environ.get(varname.strip(), "").strip()


def getenvnb(notebook, varname):
    """Get the value of zet-specific notebook environment variable."""
    notebook = notebook_env_name(notebook)
    envvar = f"ZET_NOTEBOOK_{notebook}_{varname.upper()}"
    return getenv(envvar)


def cd_root():
    os.chdir(getenv("ZET_ROOT"))


@contextlib.contextmanager
def tmpchdir(path):
    origcwd = os.getcwd()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(origcwd)


def is_in_root():
    return os.getcwd() == getenv("ZET_ROOT")


@cache
def notebook_env_name(nb):
    return nb.replace(" ", "_").upper()


def is_notebook(nb):
    return notebook_env_name(nb) in notebooks()


def notebooks():
    """Get the list of configured zet notebooks"""
    return getenv("ZET_NOTEBOOKS").split(":")


def default_notebook():
    """Return the default notebook"""
    default_file = os.path.join(getenv("ZET_ROOT"), ".default-notebook")
    try:
        with open(default_file) as f:
            nb = f.readline().strip()
            if is_notebook(nb):
                return nb
            else:
                eprint(f".default-notebook: not a notebook: {nb}")
    except FileNotFoundError:
        pass

    return notebooks()[0]


def notebook_abspath(nb):
    return os.path.join(getenv("ZET_ROOT"), getenvnb(nb, "PATH"))


def command_exists(cmd):
    """Return whether supplied command is accessible in a current environment"""
    return shutil.which(cmd) is not None


def torelpath(path):
    if path.startswith(os.getcwd()):
        return path[len(os.getcwd()) :].lstrip(os.path.sep)
    return path


def list_files(path):
    """Geneate all existing, visible (not hidden) files inside a path. If path
    is a file, it will be yielded. If it is a directory, all files inside it
    will be yielded, recursively."""

    def is_hidden(p):
        return p.startswith(".")

    if is_file(path) and os.path.exists(path):
        yield path
        return

    for root, dirs, files in os.walk(path):
        dirs[:] = [d for d in dirs if not is_hidden(d)]

        for f in files:
            if not is_hidden(f):
                yield os.path.join(root, f)


@cache
def path_of_nbpath(desc):
    """Return an absolute path of notebook description path (nbpath), which
    general form is:

        notebook:folder/subfolder/file.ext

    Each part of this description is optional. When notebook is missing, a
    default one, returned by default_notebook(), will be used.

    To differentiate directories from files without an extension, a convention
    is chosen that directories must end with a slash (see: is_file()), so:

        notebook:path   - file
        notebook:path/  - directory
    """
    notebook, _, path_inside = desc.partition(":")

    # no notebook given: some/path/file.md
    # default notebook should be used
    if not path_inside and not is_notebook(notebook):
        path_inside = notebook
        notebook = default_notebook()
    if not is_notebook(notebook):
        eprint(f"No such notebook: {notebook}. Using default one instead.")
        notebook = default_notebook()

    return os.path.join(notebook_abspath(notebook) + os.path.sep, path_inside)


def is_file(path):
    """Return a boolean telling whether path points to a file or directory.
    Path doesn't have to exist, in which case directories must end with a
    slash (see: path_of_nbpath())"""
    if os.path.exists(path):
        return not os.path.isdir(path)

    return not path.endswith(os.path.sep)


def add_basename_if_missing(path, basename):
    """Adds a basename to the path if it is recognised as a directory (via
    is_file())"""
    if not is_file(path):
        return os.path.join(path, basename)
    return path


def ensure_unique_basename(path, default_basename="note.md"):
    """Transform basename of a path in a way that makes sure that it doesn't
    exist yet."""

    assert default_basename

    path_is_file = is_file(path)
    basename = os.path.basename(path) if path_is_file else default_basename
    directory = os.path.dirname(path) if path_is_file else path

    base, ext = os.path.splitext(basename)
    abs = os.path.join(directory, basename)
    i = 0

    while os.path.exists(abs):
        i += 1
        basename = f"{base}-{i}{ext}"
        abs = os.path.join(directory, basename)

    return abs
